<?php
/*
 * @file hotel_booking.edit.inc
 * Provides editing order functions for bookings
 * @copyright Copyright(c) 2011 Lee Rowlands
 * @license GPL v2 http://www.fsf.org/licensing/licenses/gpl.html
 * @author Lee Rowlands contact at rowlandsgroup dot com
 * 
 */

/**
 * Form builder to provide the edit pane for the bookings
 * @param $order object the order object
 * @return array form array
*/
function hotel_booking_order_edit_form($order) {
  
  $form = array();
  
  $form['hotel_booking'] = array(
    '#tree' => TRUE
  );
  
  $form['hotel_booking']['products'] = array();
  hotel_booking_inc('util');
  foreach ($order->products as $key => $product) {
    if ($product->data['module'] == 'hotel_booking' &&
        is_array($product->data['nights'])) {
      $node = node_load($product->nid);
      if (node_access('update', $node)) {
        $delta = $product->order_product_id;
        $form['hotel_booking']['products'][$delta]['data'] = array(
          '#type'  => 'value',
          '#value' => serialize($product->data)
        );
        $form['hotel_booking']['products'][$delta]['room'] = array(
          '#value' => l(check_plain($product->title), 'node/'. $product->nid .'/view-availability',
                        array('attributes' => array('class' => 'lightbox-display availability',
                                                    'rel' => 'lightframe')))
        );
        list($check_in, $check_out) = _hotel_booking_check_in_out($product->data['nights']);
        $form['hotel_booking']['products'][$delta]['check_in'] = array(
          '#type' => 'date_popup',
          '#required' => TRUE,
          '#size' => 14,
          '#default_value' => $check_in->format(DATE_FORMAT_DATETIME),
          '#date_type' => DATE_ISO,
          '#date_timezone' => date_default_timezone_name(),
          '#date_format' => variable_get('uc_date_format_default', 'd M Y'),
          '#date_year_range' => '-0:+1',
        );
        $form['hotel_booking']['products'][$delta]['check_out'] = array(
          '#type' => 'date_popup',
          '#required' => TRUE,
          '#size' => 14,
          '#default_value' => $check_out->format(DATE_FORMAT_DATETIME),
          '#date_type' => DATE_ISO,
          '#date_timezone' => date_default_timezone_name(),
          '#date_format' => variable_get('uc_date_format_default', 'd M Y'),
          '#date_year_range' => '-0:+1',
        );
        $form['hotel_booking']['products'][$delta]['adults'] = array(
          '#type' => 'select',
          '#options' => drupal_map_assoc(range(1, 8)),
          '#default_value' => $product->data['adults'],
        );
        $form['hotel_booking']['products'][$delta]['children'] = array(
          '#type' => 'select',
          '#options' => range(0, 8),
          '#default_value' => $product->data['children']
        );
        $form['hotel_booking']['products'][$delta]['delete'] = array(
          '#type' => 'submit',
          '#value' => 'remove_'. $delta,
          '#submit' => array('hotel_booking_order_edit_delete_submit'), 
          '#attributes' => array('class' => 'hb-delete-item hb-icon', 'title' => t('Click here to delete this booking.')),
        );
        $form['hotel_booking']['products'][$delta]['save'] = array(
          '#type' => 'submit',
          '#value' => 'save_'. $delta,
          '#validate' => array('hotel_booking_order_edit_save_validate'),
          '#submit' => array('hotel_booking_order_edit_save_submit'),
          '#attributes' => array('class' => 'hb-save-item hb-icon', 'title' => t('Click here to save this booking.')),
        );
      }
      else {
        $form['hotel_booking']['products'][$delta]['room'] = array(
          '#value' => l(check_plain($product->title), 'node/'. $product->nid)
        );
        list($check_in, $check_out) = _hotel_booking_check_in_out($product->data['nights']);
        $form['hotel_booking']['products'][$delta]['check_in'] = array(
          '#value' => $check_in->format(variable_get('uc_date_format_default', 'd M Y'))
        );
        $form['hotel_booking']['products'][$delta]['check_out'] = array(
          '#value' => $check_out->format(variable_get('uc_date_format_default', 'd M Y'))
        );
        $form['hotel_booking']['products'][$delta]['adults'] = array(
          '#value' => $product->data['adults'],
        );
        $form['hotel_booking']['products'][$delta]['children'] = array(
          '#value' => $product->data['children']
        );
        $form['hotel_booking']['products'][$delta]['save'] = array(
          '#value' => t('N/A')
        );
        $form['hotel_booking']['products'][$delta]['delete'] = array(
          '#value' => '&nbsp;'
        );
      }
    }
  }
  
  $form['hotel_booking']['add'] = array(
    '#tree' => TRUE
  );
  
  $form['hotel_booking']['add']['room'] = array(
    '#type'          => 'textfield',
    '#size'          => 40,
    '#maxlength'     => 128,
    '#required'      => FALSE,
    '#autocomplete_path' => 'admin/store/hotel-booking/room-name',
    '#description' => t('Enter room name')
  );
      
  $form['hotel_booking']['add']['check_in'] = array(
    '#type' => 'date_popup',
    '#required' => FALSE,
    '#size' => 14,
    '#date_type' => DATE_ISO,
    '#date_timezone' => date_default_timezone_name(),
    '#date_format' => variable_get('uc_date_format_default', 'd M Y'),
    '#date_year_range' => '-0:+1',
  );
  $form['hotel_booking']['add']['check_out'] = array(
    '#type' => 'date_popup',
    '#required' => FALSE,
    '#size' => 14,
    '#date_type' => DATE_ISO,
    '#date_timezone' => date_default_timezone_name(),
    '#date_format' => variable_get('uc_date_format_default', 'd M Y'),
    '#date_year_range' => '-0:+1',
  );
  $form['hotel_booking']['add']['adults'] = array(
    '#type' => 'select',
    '#options' => drupal_map_assoc(range(1, 8)),
    '#default_value' => $product->data['adults'],
  );
  $form['hotel_booking']['add']['children'] = array(
    '#type' => 'select',
    '#options' => range(0, 8),
    '#default_value' => $product->data['children']
  );
  $form['hotel_booking']['add']['add'] = array(
    '#type' => 'submit',
    '#validate' => array('hotel_booking_order_edit_add_validate'),
    '#submit' => array('hotel_booking_order_edit_add_submit'), 
    '#value' => t('Add to booking'),
    '#attributes' => array('class' => 'hb-add-item', 'title' => t('Click here to add to this booking.')),
  );
  
  return $form;
}

/**
 * Theme callback for rendering the order pane for bookings
 * @param $form array the form object
 * @return string valid xhtml
*/
function hotel_booking_order_edit_theme($form) {
  drupal_add_css(drupal_get_path('module', 'hotel_booking') .'/css/hotel_booking.edit.css');
  drupal_add_js(drupal_get_path('module', 'hotel_booking') .'/js/hotel_booking.edit.js');
  
  $rows = array();
  $header = array(
    t('Room'),
    t('Check-In'),
    t('Check-Out'),
    t('Adults'),
    t('Children'),
    t('Ops')
  );
  foreach (element_children($form['hotel_booking']['products']) as $delta) {
    $row = array(
      array('data' => drupal_render($form['hotel_booking']['products'][$delta]['room']),
            'class' => 'hb-edit-room'),
      array('data' => drupal_render($form['hotel_booking']['products'][$delta]['check_in']),
            'class' => 'hb-edit-check-in'),
      array('data' => drupal_render($form['hotel_booking']['products'][$delta]['check_out']),
            'class' => 'hb-edit-check-out'),
      array('data' => drupal_render($form['hotel_booking']['products'][$delta]['adults']),
            'class' => 'hb-edit-adults'),
      array('data' => drupal_render($form['hotel_booking']['products'][$delta]['children']),
            'class' => 'hb-edit-children'),
      array('data' => drupal_render($form['hotel_booking']['products'][$delta]['save']) . drupal_render($form['hotel_booking']['products'][$delta]['delete']),
            'class' => 'hb-edit-ops'),
    );
    $rows[] = $row;
  }
  if (!count($rows)) {
    $rows[] = array(array('data' => t('There are no bookings in this order yet.'), 'colspan' => 6));
  }
  $rows[] = array(array('data' => t('Add a booking.'),
                        'class' => 'hb-edit-add-header',
                        'colspan' => 6));
  $row = array(
    array('data' => drupal_render($form['hotel_booking']['add']['room']),
          'class' => 'hb-edit-room'),
    array('data' => drupal_render($form['hotel_booking']['add']['check_in']),
          'class' => 'hb-edit-check-in'),
    array('data' => drupal_render($form['hotel_booking']['add']['check_out']),
          'class' => 'hb-edit-check-out'),
    array('data' => drupal_render($form['hotel_booking']['add']['adults']),
          'class' => 'hb-edit-adults'),
    array('data' => drupal_render($form['hotel_booking']['add']['children']),
          'class' => 'hb-edit-children'),
    array('data' => drupal_render($form['hotel_booking']['add']['add']),
          'class' => 'hb-edit-ops'),
  );
  $rows[] = $row;
  $output = theme('table', $header, $rows, array('class' => 'hb-edit'));
  return $output . drupal_render($form['hotel_booking']);
}

/**
 * Process callback for changes to the bookings order
 * @param $values array of form_values
 * @return array of changes
*/
function hotel_booking_order_edit_process($values) {
  dsm($values);
  dsm($values['hotel_booking']);
  dsm($values['op']);
  // We have to update the products and return those
  // so $order->products has our new values
  if ($values['op'] == t('Add to booking')) {
    
  }
  return array();
}

/**
 * Validate selections for adding a booking
*/
function hotel_booking_order_edit_add_validate($form, &$form_state) {
  $values = $form_state['values'];
  if (empty($values['hotel_booking']['add']['room'])) {
    form_set_error('hotel_booking][add][room', t('You must enter a room name'));
  }
  else {
    $result = preg_match_all('/ \[id:([0-9]+)\]/', $values['hotel_booking']['add']['room'], $matches);
    if (isset($matches[1][0]) &&
        is_numeric($matches[1][0])) {
      $node = node_load($matches[1][0]);
      if (!$node || !user_access('update', $node) || $node->type != 'hotel_room_type') {
        form_set_error('hotel_booking][add][room', t('There is no such room or you do not have permission to edit bookings for that room.'));
      }
    }
    else {
      $node = node_load(array('type' => 'hotel_room_type', 'title' => $values['hotel_booking']['add']['room']));
      if (!$node || !user_access('update', $node)) {
        form_set_error('hotel_booking][add][room', t('There is no such room or you do not have permission to edit bookings for that room.'));
      }
    }
  }
  if (empty($values['hotel_booking']['add']['check_in'])) {
    form_set_error('hotel_booking][add][check_in', t('You must enter a check in date'));
  }
  if (empty($values['hotel_booking']['add']['check_out'])) {
    form_set_error('hotel_booking][add][check_out', t('You must enter a check out date'));
  }
  if ($node) {
    $errors = hotel_booking_edit_check_valid($node, $values['hotel_booking']['add']['check_in'],
                                             $values['hotel_booking']['add']['check_out'],
                                             $values['hotel_booking']['add']['adults'],
                                             $values['hotel_booking']['add']['children']);
    if (count($errors)) {
      foreach ($errors as $field => $message) {
        form_set_error('hotel_booking][add]['. $field, $message);
      }
    }
  }
}

/**
 * Submit handler for adding a new item
*/
function hotel_booking_order_edit_add_submit($form, &$form_state) {
  $order = uc_order_load($form_state['values']['order_id']);
  $node = node_load($original_product->nid);
  $values = $form_state['values']['hotel_booking']['add'];
  $result = preg_match_all('/ \[id:([0-9]+)\]/', $values['room'], $matches);
  if (isset($matches[1][0]) &&
      is_numeric($matches[1][0])) {
    $node = node_load($matches[1][0]);
  }
  else {
    $node = node_load(array('type' => 'hotel_room_type', 'title' => $values['room']));
  }
  if (!$node) {
    drupal_set_message(t('Could not find a room by that name.'), 'error');
    return;
  }
  // First we calculate the prices
  $prices = hotel_booking_calculate_prices(
    $values['check_in'],
    $values['check_out'],
    array($node->nid),
    $values['adults'],
    $values['children'],
    3 //no smoking preference
  );
  // Then we create our new product and save it
  $total = $prices['totals'][$node->nid];
  $breakdown = $prices['results'][$node->nid]['prices'];

  $nights = $rates = array();
  //nightly breakdown
  foreach ($breakdown as $date => $price) {
    $nights[] = $date;
    $rates[]= $price;
  }
  $data = array(
    'module' => 'hotel_booking',
    'prices' => $rates,
    'nights' => $nights,
    'total' => $total,
    'adults' => $values['adults'],
    'children' => $values['children'],
  );
  $product = drupal_clone($node);
  $product->data = $data;
  $product->price = $total;
  $product->cost = $product->weight = 0;
  $product->qty = 1;
  
  // Add the product to our order
  $order->products[] = $product;
  
  // Save the order with the new products
  uc_order_save($order);
  drupal_set_message(t('Added your booking item'));
  // Now we make a dummy order object to handle the availability updates
  $dummy_order = (object)array(
    'products' => array($product),
    'status' => 'processing',
  );
  // Then we update availability
  hotel_booking_alter_availability($dummy_order);
}

/**
 * Validate selections for editing a booking
*/
function hotel_booking_order_edit_save_validate($form, &$form_state) {
  $order_product_id = $form_state['clicked_button']['#parents'][2];
  $values = $form_state['values']['hotel_booking']['products'][$order_product_id];
  if (empty($values)) {
    form_set_error(' ', t('An error occured whilst processing your request, please try again'));
    return;
  }
  $original_product = db_fetch_object(db_query("SELECT * FROM {uc_order_products}
                                       WHERE order_product_id = %d", $order_product_id));
  $node = node_load($original_product->nid);
  $original_data = unserialize($values['data']);
  if (empty($values['check_in'])) {
    form_set_error('hotel_booking][products]['. $order_product_id .'][check_in', t('You must enter a check in date'));
  }
  if (empty($values['check_out'])) {
    form_set_error('hotel_booking][products]['. $order_product_id .'][check_out', t('You must enter a check out date'));
  }
  hotel_booking_inc('util');
  list($original_check_in, $original_check_out) = _hotel_booking_check_in_out($original_data['nights']);
  $errors = hotel_booking_edit_check_valid($node, $values['check_in'],
                                           $values['check_out'],
                                           $values['adults'],
                                           $values['children'],
                                           $original_check_in->format(DATE_FORMAT_DATETIME),
                                           $original_check_out->format(DATE_FORMAT_DATETIME));
  if (count($errors)) {
    foreach ($errors as $field => $message) {
      form_set_error('hotel_booking][products]['. $order_product_id .']['. $field, $message);
    }
  }
}

/**
 * Submit handler for saving changes to an item
*/
function hotel_booking_order_edit_save_submit($form, &$form_state) {
  $order_product_id = $form_state['clicked_button']['#parents'][2];
  $values = $form_state['values']['hotel_booking']['products'][$order_product_id];
  if (empty($values)) {
    form_set_error(' ', t('An error occured whilst processing your request, please try again'));
    return;
  }
  $original_product = db_fetch_object(db_query("SELECT * FROM {uc_order_products}
                                       WHERE order_product_id = %d", $order_product_id));
  $original_product->data = unserialize($original_product->data);
  $order = uc_order_load($original_product->order_id);
  $node = node_load($original_product->nid);
  // First we update the old availability by generating a dummy order
  $dummy_order = (object)array(
    'products' => array($original_product),
    'status' => 'canceled',
  );
  hotel_booking_alter_availability($dummy_order);
  // Then we calculate the prices
  $prices = hotel_booking_calculate_prices(
    $values['check_in'],
    $values['check_out'],
    array($node->nid),
    $values['adults'],
    $values['children'],
    3 //no smoking preference
  );
  // Then we add our new data and save it
  $total = $prices['totals'][$node->nid];
  $breakdown = $prices['results'][$node->nid]['prices'];

  $nights = $rates = array();
  //nightly breakdown
  foreach ($breakdown as $date => $price) {
    $nights[] = $date;
    $rates[]= $price;
  }
  $new_data = array(
    'prices' => $rates,
    'nights' => $nights,
    'total' => $total,
    'adults' => $values['adults'],
    'children' => $values['children'],
  );
  foreach ($new_data as $key => $value) {
    $original_product->data[$key] = $value;
  }
  $original_product->price = $total;
  db_query("UPDATE {uc_order_products} SET data = '%s'
           WHERE order_product_id = %d", serialize($original_product->data), $order_product_id);
  // Then we update our order object and save that
  foreach ($order->products as $ix => $product) {
    if ($product->order_product_id == $original_product->order_product_id) {
      $order->products[$ix] = $original_product;
    }
  }
  // Save the order with the new products
  uc_order_save($order);
  drupal_set_message(t('Updated your booking item'));
  // Now we make a dummy order object to handle the availability updates
  $dummy_order = (object)array(
    'products' => array($original_product),
    'status' => 'processing',
  );
  // Then we update availability
  hotel_booking_alter_availability($dummy_order);
}



/**
 * Title callback for rendering the order pane for bookings
 * @param $form array the form object
 * @return string valid xhtml
*/
function hotel_booking_order_edit_title($form) {
  return t('Edit Bookings');
}

/**
 * Autocomplete callback for room titles
 * @param $string string string to search for
*/
function hotel_booking_room_autocomplete($string = FALSE) {
  if (!$string) {
    echo drupal_to_js(array());
    return;
  }
  $res = db_query(db_rewrite_sql("SELECT nid, title, language
                         FROM {node} n
                         WHERE type = 'hotel_room_type'
                         AND status = 1
                         AND title LIKE '%%%s%%'"), $string);
  
  $counter = 0;
  $matches = array();
  while ($room = db_fetch_object($res)) {
    if ($counter < 5) {
      $node = node_load($room->nid);
      if (node_access('update', $node)) {
        $matches[check_plain($room->title) .' [id:'. $room->nid .']'] = check_plain($room->title);
        $counter++;
      }
    }
  }
  echo drupal_to_js($matches);
  return;
}

/**
 * Validation handler to ensure a booking edited/added is valid
 * @param $room object room node
 * @param $check_in string check in date
 * @param $check_out string check out date
 * @param $adults int # of adults
 * @param $children int #number of children
 * @param $original_check_in string original date
 * @param $original_check_out string original date
 * @return array of errors - if empty, edit is valid
*/
function hotel_booking_edit_check_valid($room, $check_in, $check_out, $adults, 
                                        $children, $original_check_in = FALSE,
                                        $original_check_out = FALSE) {
  $errors = array();
  $visitors = $adults + $children;
  // Check visitors ok against capacity.
  if ($room->capacity < $visitors) {
    $errors['adults'] = t('This room can only accommodate !capacity occupants',
                               array('!capacity' => $room->capacity));
  }
  
  // Now we only need to check availablity if dates have changed.
  if ($original_check_in && $original_check_out &&
      ($original_check_in != $check_in ||
       $original_check_out != $check_out)) {
    hotel_booking_inc('util');
    
    if ($check_out != $original_check_out) {
      // If later check out, we need to calculate based on original checkout
      // i.e. we are searching from their original checkout to the new checkout.
      
      $addtl_check_in = $original_check_out;
      $addtl_check_in_date = date_make_date($addtl_check_in);
      $check_out_date = date_make_date($check_out);
    
      $new_nights = ((int)$check_out_date->format('U') - (int)$addtl_check_in_date->format('U'))/ 86400; //86400 secs in day
      // Fetch rooms.
      $rooms = hotel_booking_get_available_rooms(
        $addtl_check_in,
        $check_out,
        $new_nights,
        $adults,
        $children,
        3, // No smoking preference
        array($room->nid)
      );
    
      drupal_alter('hotel_booking_available_rooms', $rooms, array(
        'check_in' => $addtl_check_in,
        'check_out' => $check_out,
        'nights' => $nights,
        'adults' => $adults,
        'children' => $children,
        'smoking' => 3 // No prefernce
      ), TRUE);
    
      if (count($rooms) == 0) {
        //the room is not available
        $errors['room'] = t('This room is not available for those dates');
      }
    }
    
    if ($check_in != $original_check_in) {
      // If earlier check in, we need to calculate based on original checkin
      // i.e. we are searching from their new checkin to the original checkin.
      
      $addtl_check_out = $original_check_in;
      $addtl_check_out_date = date_make_date($addtl_check_out);
      $check_in_date = date_make_date($check_in);
    
      $new_nights = ((int)$addtl_check_out_date->format('U') - (int)$check_in_date->format('U'))/ 86400; //86400 secs in day
      // Fetch rooms.
      $rooms = hotel_booking_get_available_rooms(
        $check_in,
        $addtl_check_out,
        $new_nights,
        $adults,
        $children,
        3, // No smoking preference
        array($room->nid)
      );
    
      drupal_alter('hotel_booking_available_rooms', $rooms, array(
        'check_in' => $check_in,
        'check_out' => $addtl_check_out,
        'nights' => $nights,
        'adults' => $adults,
        'children' => $children,
        'smoking' => 3 // No prefernce
      ), TRUE);
    
      if (count($rooms) == 0) {
        //the room is not available
        $errors['room'] = t('This room is not available for those dates');
      }
    }
  }
  
  // New bookings
  if (!$original_check_in) {
    $check_out_date = date_make_date($check_out);
    $check_in_date = date_make_date($check_in);
  
    $nights = ((int)$check_out_date->format('U') - (int)$check_in_date->format('U'))/ 86400; //86400 secs in day
    // Fetch rooms.
    $rooms = hotel_booking_get_available_rooms(
      $check_in,
      $check_out,
      $nights,
      $adults,
      $children,
      3, // No smoking preference
      array($room->nid)
    );
  
    drupal_alter('hotel_booking_available_rooms', $rooms, array(
      'check_in' => $check_in,
      'check_out' => $check_out,
      'nights' => $nights,
      'adults' => $adults,
      'children' => $children,
      'smoking' => 3 // No prefernce
    ), TRUE);
  
    if (count($rooms) == 0) {
      //the room is not available
      $errors['room'] = t('This room is not available for those dates');
    }
  }
  return $errors;
}

/**
 * Form builder to ensure confirmation of booking deletion
 *
*/
function hotel_booking_edit_confirm_delete($form_state, $order_product) {
  $form = array();
  $form['order_product'] = array(
    '#type'  => 'value',
    '#value' => $order_product
  );
  return confirm_form($form,
    t('Are you sure you want to remove this booking?'),
    $_GET['destination']
  );  
}

/**
 * Submit handler for removing a booking from an order
 * 
*/
function hotel_booking_edit_confirm_delete_submit($form, &$form_state) {
  $order_product = $form_state['values']['order_product'];
  $order = uc_order_load($order_product->order_id);
  if ($order && is_array($order->products)) {
    foreach ($order->products as $ix => $product) {
      if ($product->order_product_id == $order_product->order_product_id) {
        unset($order->products[$ix]);
      }
    }
    // Save the order with the new products
    db_query("DELETE FROM {uc_order_products} WHERE order_product_id = %d", $order_product->order_product_id);
    uc_order_save($order);
    drupal_set_message(t('Deleted your booking item'));
    // Now we make a dummy order object to handle the availability updates
    $dummy_order = (object)array(
      'products' => array($order_product),
      'status' => 'canceled',
    );
    hotel_booking_alter_availability($dummy_order);
  }
}

/**
 * Submission handler to handle redirect and removal of order product
*/
function hotel_booking_order_edit_delete_submit($form, &$form_state) {
  $order_product_id = $form_state['clicked_button']['#parents'][2];
  $values = $form_state['values']['hotel_booking']['products'][$order_product_id];
  if (empty($values)) {
    form_set_error(' ', t('An error occured whilst processing your request, please try again'));
    return;
  }
  drupal_goto('admin/store/hotel-booking/delete-item/'. $order_product_id, drupal_get_destination());
}

